#!/usr/bin/env python3
#

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information


from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Age,
    Alternative,
    Integer,
    ListOfStrings,
    Percentage,
    Tuple,
)

from cmk.gui.plugins.wato import (
    CheckParameterRulespecWithItem,
    CheckParameterRulespecWithoutItem,
    rulespec_registry,
    RulespecGroupCheckParametersStorage,
)

def _item_spec_powerscale_quota():
    return TextAscii(title=_("Dell PowerScale SmartQuota"))


def _parameter_valuespec_powerscale_quota():
    return Dictionary(
        title    = _("Dell PowerScale SmartQuota"),
        help     = _("Activate special agent powerscale to use this."),
        elements = [
            ("upper_levels",
                Tuple(
                    title    = _("Upper levels"),
                    help     = _("Upper levels for usage. Set to 0 to disable checking."),
                    elements = [
                        Percentage(
                            title         = _("Warning if above"),
                            unit          = _("%"),
                            default_value = 95
                        ),
                        Percentage(
                            title         = _("Critical if above"),
                            unit          = _("%"),
                            default_value = 99
                        )
                    ]
                )
            ),
        ],
    )

rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="powerscale_quotas",
        group=RulespecGroupCheckParametersApplications,
        item_spec=_item_spec_powerscale_quota,
        parameter_valuespec=_parameter_valuespec_powerscale_quota,
        title=lambda: _("Dell PowerScale SmartQuota"),
    )
)


def _item_spec_powerscale_synciq():
    return TextAscii(title=_("Dell PowerScale SyncIQ"))

def _parameter_valuespec_powerscale_synciq():
    return Dictionary(
        title    = _("Dell PowerScale SyncIQ"),
        help     = _("Activate special agent powerscale to use this."),
        elements = [
            ("levels_age",
                Tuple(
                    title    = _("Age of last Sync"),
                    help     = _("Age of last synchronization. Set to 0 to disable checking."),
                    elements = [
                        Age(
                            title         = _("Warning if older"),
                            default_value = 90000
                        ),
                        Age(
                            title         = _("Critical if older"),
                            default_value = 176400
                        )
                    ]
                )
            ),
            ("ok_states",
                 ListOfStrings(
                     title = "SyncIQ states considered OK",
                     default_value = ['finished', 'running', 'scheduled'],
                     help = _("states that are OK."),
                 )
            ),

        ],
    )

rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="powerscale_synciq",
        group=RulespecGroupCheckParametersApplications,
        item_spec=_item_spec_powerscale_synciq,
        parameter_valuespec=_parameter_valuespec_powerscale_synciq,
        title=lambda: _("Dell PowerScale SyncIQ"),
    )
)


def _parameter_valuespec_powerscale_events():
    return Dictionary(
        title    = _("Dell PowerScale Eventlog"),
        help     = _("Activate special agent powerscale to use this."),
        elements = [
            ("crit_states",
                 ListOfStrings(
                     title = "Event states considered CRIT",
                     default_value = ['critical', 'emergency'],
                     help = _("states that are CRIT. Any other unresolved event is WARN."),
                 )
            ),
            ("ignore_list",
                 ListOfStrings(
                     title = "Ignored Events",
                     default_value = ['Security Verification Successfully Ran.', 
                                      'Security verification check failed'],
                     help = _("Enter Event group description."),
                 )
            ),

        ],
    )

rulespec_registry.register(
    CheckParameterRulespecWithoutItem(
        check_group_name="powerscale_events",
        group=RulespecGroupCheckParametersApplications,
        parameter_valuespec=_parameter_valuespec_powerscale_events,
        title=lambda: _("Dell PowerScale Eventlog"),
    )
)


def _item_spec_powerscale_nodes():
    return TextAscii(title=_("Dell PowerScale Node"))

def _parameter_valuespec_powerscale_nodes():
    return Dictionary(
        title    = _("Dell PowerScale Nodes"),
        help     = _("Activate special agent powerscale to use this."),
        elements = [
            ("upper_levels_Fans",
                Tuple(
                    title    = _("Upper levels FANs"),
                    help     = _("Upper levels for rpm. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'rpm',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'rpm',
                            default_value = 0
                        )
                    ]
                )
            ),
            ("upper_levels_fs",
                Tuple(
                    title    = _("Upper levels Filesystems"),
                    help     = _("Upper levels for Filesystems. Set to 0 to disable checking."),
                    elements = [
                        Percentage(
                            title         = _("Warning if above"),
                            unit          = '%',
                            default_value = 75.0
                        ),
                        Percentage(
                            title         = _("Critical if above"),
                            unit          = '%',
                            default_value = 90.0
                        )
                    ]
                )
            ),
            ("upper_levels_Temps",
                Tuple(
                    title    = _("Upper levels Temperature"),
                    help     = _("Upper levels for Temperature. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = '°C',
                            default_value = 70
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = '°C',
                            default_value = 80
                        )
                    ]
                )
            ),
            ("ok_states_Battery",
                 ListOfStrings(
                     title         = "Battery status considered OK",
                     default_value = ['passed'],
                     help          = _("States that are OK. Any other is CRIT."),
                 )
            ),
            ("ok_states_Disk",
                 ListOfStrings(
                     title         = "Disk status considered OK",
                     default_value = ['HEALTHY', 'L3'],
                     help          = _("States that are OK. Any other is CRIT."),
                 )
            ),
        ],
    )

rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name="powerscale_nodes",
        group=RulespecGroupCheckParametersApplications,
        item_spec=_item_spec_powerscale_nodes,
        parameter_valuespec=_parameter_valuespec_powerscale_nodes,
        title=lambda: _("Dell PowerScale Nodes"),
    )
)


def _parameter_valuespec_powerscale_system():
    return Dictionary(
        title    = _("Dell PowerScale Statistics"),
        help     = _("Activate special agent powerscale to use this."),
        elements = [
            ("upper_levels_cpu",
                Tuple(
                    title    = _("CPU usage"),
                    help     = _("Upper levels for CPU. Set to 0 to disable checking."),
                    elements = [
                        Percentage(
                            title         = _("Warning if above"),
                            unit          = '%',
                            default_value = 0.0
                        ),
                        Percentage(
                            title         = _("Critical if above"),
                            unit          = '%',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_net",
                Tuple(
                    title    = _("Network bandwidth"),
                    help     = _("Upper levels for Network. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_disk",
                Tuple(
                    title    = _("Disk bandwidth"),
                    help     = _("Upper levels for Disk. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_ftp",
                Tuple(
                    title    = _("FTP bandwidth"),
                    help     = _("Upper levels for FTP. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_hdfs",
                Tuple(
                    title    = _("HDFS bandwidth"),
                    help     = _("Upper levels for HDFS. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_http",
                Tuple(
                    title    = _("HTTP bandwidth"),
                    help     = _("Upper levels for HTTP. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_nfs",
                Tuple(
                    title    = _("NFS bandwidth"),
                    help     = _("Upper levels for NFS. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_s3",
                Tuple(
                    title    = _("S3 bandwidth"),
                    help     = _("Upper levels for S3. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
            ("upper_levels_smb",
                Tuple(
                    title    = _("SMB bandwidth"),
                    help     = _("Upper levels for SMB. Set to 0 to disable checking."),
                    elements = [
                        Integer(
                            title         = _("Warning if above"),
                            unit          = 'Bytes/s',
                            default_value = 0
                        ),
                        Integer(
                            title         = _("Critical if above"),
                            unit          = 'Bytes/s',
                            default_value = 0.0
                        )
                    ]
                )
            ),
        ],
    )

rulespec_registry.register(
    CheckParameterRulespecWithoutItem(
        check_group_name="powerscale_system",
        group=RulespecGroupCheckParametersApplications,
        parameter_valuespec=_parameter_valuespec_powerscale_system,
        title=lambda: _("Dell PowerScale Eventlog"),
    )
)

