#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-


# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information


#<<<powerscale_synciq>>>
#policy_name mypolicy1              <- new policy
#end_time 1680011199
#start_time 1680011101
#state finished
#total_network_bytes 953075
#policy_name mypolicy2              <- new policy
#end_time 1680005805
#start_time 1680005701
#state finished
#total_network_bytes 13905
#policy_name mypolicy3              <- new policy
#...


# import api
from .agent_based_api.v1 import (
    Metric,
    register,
    render,
    Result,
    State,
    Service,
)

import time

# factory settings:
powerscale_synciq_factory_settings = {
    'levels_age'     :  (90000, 176400), # 25/49h
    'ok_states'      :  ['finished', 'running', 'scheduled'],
}

# parse function
def parse_powerscale_synciq(string_table):

    # nothing to do, maybe wrong privs
    if len(string_table) == 0:
        return

    # we'll return this later
    data              = []
    show_summary      = False

    # loop thru
    for line in string_table:

        # summary desired
        if line[0] == 'show_summary':
            show_summary = True
            continue

        # a new item
        if line[0] == 'policy_name':

            # finishing previous item
            try:
                data.append(entry)

            # expected to fail on very first run
            except:
                pass

            entry    = {}
            entry['show_summary'] = show_summary
            entry['policy_name'] = line[1]
            continue

        # all the info for item
        entry[line[0]] = line[1]
    

    # last run. entry is empty here on no synciq configured _and_ summary requested
    try:
        data.append(entry)
    except:
        return

    # sort by age
    sortdata = sorted(data, key=lambda k : k['end_time'])
    return sortdata


# discovery function
def discover_powerscale_synciq(section):

    for policy in list(section):
        if policy['show_summary'] == True:
            yield Service(item='Summary', parameters={'expected' : len(section)})
            return
        yield Service( item=policy['policy_name'])


# check function
def check_powerscale_synciq(item, params, section):

    # initial
    mysummary    = []
    mydetails    = []

    warn, crit   = params['levels_age']

    # summary
    if item == 'Summary':

        for policy in list(section):

            # state
            if policy['state'] not in params['ok_states']:
                info = '%s: state %s (!)' % (policy['policy_name'], policy['state'])
                mysummary.append(info)
                mydetails.append(info)

            # age
            age = int(time.time()-int(policy['end_time']))

            # tell the others
            info = '%s: last sync %s ago' % (policy['policy_name'], render.timespan(age))

            # crit to summary and details
            if age > crit and crit > 0:
                mysummary.append('%s(!!)' % info)
                mydetails.append('%s(!!)' % info)
                
            # warn to summary and details
            elif age > warn and crit > 0:
                mysummary.append('%s(!)' % info)
                mydetails.append('%s(!)' % info)

            # OK to details
            else:
                mydetails.append(info)


        # everything is OK:
        l = len(mysummary)
        t = len(section)
        if l == 0:
            mysummary.append('%s SyncIQ OK' % t)

        if l == 1 or l == 2:
            o = t - l
            mysummary.append('%s SyncIQ OK' % o)

        # rewrite summary if more than 2 warn/crit
        if l > 2:
            m = '(!!)' if '!!' in ' '.join(mysummary) else '(!)'
            mysummary  = []
            mysummary.append('%s (of total %s) SyncIQ not OK%s' % (l, t, m))

        # count
        e = params['expected']
        if t != e:
            mysummary.append('%s total SyncIQ(!) (expected %s)' % (t, e))

        mydetails = '\n'.join(mydetails)

    # single policy
    else:

        # find my policy in list
        for policy in list(section):
            if policy['policy_name'] == item:
                data = policy
                break

        # check for existence
        try:
            data['policy_name']
        except:
            yield Result(state=State.UNKNOWN, summary='not found')
            return

        # state
        if policy['state'] not in params['ok_states']:
            mysummary.append('state %s (!)' % policy['state'])

        # age
        age = int(time.time()-int(policy['end_time']))

        # check
        m = '(!!)' if age > crit and crit > 0 else '(!)' if age > warn and crit > 0 else ''
        mysummary.append('last sync %s ago%s' % (render.timespan(age), m))

        # Metric
        yield Metric('age', age, levels=(warn, crit))
                

    # get worst state from markers
    mysummary = ', '.join(mysummary)
    mystate   = State.CRIT if '!!' in mysummary else State.WARN if '!' in mysummary else State.OK

    # this is the end, my friend
    try:
        yield Result(state=mystate, summary=mysummary, details=mydetails)
    except:
        yield Result(state=mystate, summary=mysummary)
    return

        

# register parse function
register.agent_section(
    name                = "powerscale_synciq",
    parse_function      = parse_powerscale_synciq,
)

# register check
register.check_plugin(
    name                     = "powerscale_synciq",
    service_name             = "SyncIQ %s",
    discovery_function       = discover_powerscale_synciq,
    check_function           = check_powerscale_synciq,
    check_ruleset_name       = "powerscale_synciq",
    check_default_parameters = powerscale_synciq_factory_settings,
)
